@@ -46,31 +46,27 @@ to print the corresponding values, or T for all of them.")
   (dolist (sym (apropos-list string package))
     (format t "~&~s~40t" sym)
     (when (fboundp sym)
-      (write-string "   ")
+      #1=(write-string "   ")   ; spacer
       (write-string (fbound-string sym))
       (when (apropos-do-more :function)
         (format t " [~s]" (fdefinition sym))))
     (when (symbol-macro-expand sym)
-      (write-string "   ")
-      (write-string (TEXT "symbol-macro")))
+      #1# (write-string (TEXT "symbol-macro")))
     (when (boundp sym)
-      (write-string "   ")
-      (if (constantp sym)
-        (write-string (TEXT "constant"))
-        (write-string (TEXT "variable")))
+      #1# (if (constantp sym)
+            (write-string (TEXT "constant"))
+            (write-string (TEXT "variable")))
       (when (apropos-do-more :variable)
         (format t " [~s]" (symbol-value sym))))
     (let ((type (or (get sym 'system::type-symbol)
                     (get sym 'system::defstruct-description))))
       (when type
-        (write-string "   ")
-        (write-string (TEXT "type"))
+        #1# (write-string (TEXT "type"))
         (when (apropos-do-more :type)
           (format t " [~s]" type))))
     (let ((class (get sym 'clos::closclass)))
       (when class
-        (write-string "   ")
-        (write-string (TEXT "class"))
+        #1# (write-string (TEXT "class"))
         (when (apropos-do-more :class)
           (format t " [~s]" class))))
     (elastic-newline))
@@ -105,6 +101,14 @@ to print the corresponding values, or T for all of them.")
         (terpri stream)
         (format stream (TEXT "No slots."))))))
 
+(defun launch-doc (obj type stream name)
+  (when (zerop *describe-nesting*)
+    (let ((doc (documentation obj type)))
+      (when doc
+        (if *browser*
+            (ext::browse-url doc :out stream)
+            (format stream name doc))))))
+
 (clos:defgeneric describe-object (obj stream)
   (:method ((obj t) (stream stream))
     (ecase (type-of obj)
@@ -209,6 +213,10 @@ to print the corresponding values, or T for all of them.")
     (format stream (TEXT "an instance of the CLOS class ~S.")
             (clos:class-of obj))
     (describe-slotted-object obj stream))
+  (:method ((obj clos:funcallable-standard-object) (stream stream))
+    (format stream (TEXT "an instance of the CLOS class ~S, can be used as a function.")
+            (clos:class-of obj))
+    (describe-slotted-object obj stream))
   (:method ((obj structure-object) (stream stream)) ; CLISP specific
     (format stream (TEXT "a structure of type ~S.")
             (type-of obj))
@@ -317,6 +325,10 @@ to print the corresponding values, or T for all of them.")
             (format stream (TEXT "Documentation as a ~a:") ty)
             (terpri stream)
             (princ doc stream))))
+      (launch-doc obj 'ext::clhs ; change to sys::clhs when ext:clhs is finally removed
+                  stream (TEXT "~%ANSI Documentation is at~% ~S"))
+      (launch-doc obj 'sys::impnotes stream
+                  (TEXT "~%CLISP Documentation is at~% ~S"))
       (when moree
         (terpri stream)
         (format stream (TEXT "For more information, evaluate ~{~S~^ or ~}.")
@@ -416,7 +428,9 @@ to print the corresponding values, or T for all of them.")
                      (if case-sensitive-p
                        (format stream (TEXT "It is case-sensitive, but not case-inverted!"))
                        (format stream (TEXT "It is a traditional ANSI CL compatible package, but uses the symbols from the modern ~S!")
-                               (find-package "CS-COMMON-LISP")))))))))
+                               (find-package "CS-COMMON-LISP"))))))))
+        (launch-doc obj 'sys::impnotes stream
+                    (TEXT "~%CLISP Documentation is at~% ~S")))
       (format stream (TEXT "a deleted package."))))
   (:method ((obj hash-table) (stream stream))
     (let ((count (hash-table-count obj)))
@@ -496,7 +510,7 @@ to print the corresponding values, or T for all of them.")
                             (clos:method-specializers meth)))))
         (format stream (TEXT "No methods.")))))
   (:method ((obj function) (stream stream))
-    (ecase (type-of obj)
+    (ecase (type-of obj)        ; not etypecase!
       #+FFI
       (FFI::FOREIGN-FUNCTION
        (format stream (TEXT "a foreign function of foreign type ~S.")
@@ -528,6 +542,8 @@ to print the corresponding values, or T for all of them.")
                            (TEXT "For more information, evaluate ~{~S~^ or ~}.")
                            `((DISASSEMBLE ,funform))))))))))
       (FUNCTION
+       ;; we do not use ETYPECASE here to ensure that if we do get here,
+       ;; we are dealing with an Iclosure object
        (format stream (TEXT "an interpreted function."))
        (let ((doc (sys::%record-ref obj 2)))
          (terpri stream)
@@ -746,10 +762,10 @@ to print the corresponding values, or T for all of them.")
         (assert (= (parse-integer (nth 0 pieces) :radix 16) (char-code ch)))
         (values-list
          (append
-          (mapcar #'(lambda (x) (if (equal x "") nil x))
+          (mapcar #'(lambda (x) (if (equal x #1="") nil x))
                   (subseq pieces 1 12))
           (mapcar #'(lambda (x)
-                      (if (equal x "") nil
+                      (if (equal x #1#) nil
                           (code-char (parse-integer x :radix 16))))
                   (subseq pieces 12 15))))))))
 
