@@ -79,11 +79,11 @@ The input file is normal C code, modified like this:
       if (a == c_name_table[index].c_const)
         return *c_name_table[index].l_const;
     if (a == default) return NIL;
-    NOTREACHED; /* or "return reverse(a);" if reverse is supplied */
+    ERROR; /* or "return reverse(a);" if reverse is supplied */
   }
  enum means no #ifdef
  prefix, suffix default to ""
- reverse defaults to "" and means NOTREACHED
+ reverse defaults to "" and means ERROR
  bitmasks means additional *_to_list and *_from_list functions are defined
  delim defaults to "_" and separates prefix and suffix
 
@@ -950,7 +950,7 @@ commas and parentheses."
     (formatln out "#define F(varname) subr_tab_ptr_as_object(&(~a._##varname))"
               subr-tab)
     (newline out)
-    (formatln out "struct {")
+    (formatln out "struct ~A_t {" object-tab)
     (setq *objdefs* (sort *objdefs* #'string-lessp :key #'objdef-tag)
           *fundefs* (sort *fundefs* #'string-lessp :key #'fundef-tag))
     (loop :for od :across *objdefs*
@@ -960,11 +960,10 @@ commas and parentheses."
       :do (with-conditional (out (vardef-cond-stack vd))
             (format out "  gcv_object_t _~A;" (vardef-tag vd))))
     (formatln out "} ~A;" object-tab)
-    (formatln
-     out "uintC module__~A__object_tab_size = sizeof(~A)/sizeof(gcv_object_t);"
-     *module-name* object-tab)
+    (formatln out "uintC ~A_size = sizeof(~A)/sizeof(gcv_object_t);"
+              object-tab object-tab)
     (newline out)
-    (formatln out "struct {")
+    (formatln out "struct ~A_t {" object-tab-initdata)
     (loop :for od :across *objdefs*
       :do (with-conditional (out (objdef-cond-stack od))
             (format out "  object_initdata_t _~A;" (objdef-tag od))))
@@ -1020,89 +1019,41 @@ commas and parentheses."
       :for reverse = (checker-reverse ch) :for bitmasks = (checker-bitmasks ch)
       :for type-tag = (objdef-tag (checker-type-odef ch))
       :for c-name = (checker-name ch) :for c-type = (or (checker-type ch) "int")
-      :for enum-p = (checker-enum-p ch) :for need-default = (stringp default)
-      :initially
-      (formatln out "struct c_lisp_pair {int c_const; gcv_object_t *l_const;};")
-      (newline out)
+      :for enum-p = (checker-enum-p ch)
       :do (with-conditional (out (checker-cond-stack ch))
-            (formatln out "static struct c_lisp_pair ~A_table[] = {" c-name)
+            (formatln out "static const c_lisp_pair_t ~A_table[] = {" c-name)
             (loop :for name :in (checker-cpp-names ch)
               :for name-C = (to-C-name name prefix suffix delim)
               :for odef :in (checker-cpp-odefs ch)
               :for need-ifdef = (and (not enum-p) (stringp name))
               :do (when need-ifdef (formatln out " #ifdef ~A" name-C))
               (formatln out "  { ~A, &(O(~A)) }," name-C (objdef-tag odef))
-              (when (and need-default (string= default name-C))
-                (setq need-default nil))
               (when need-ifdef (formatln out " #endif")))
             (formatln out "  { 0, NULL }")
             (formatln out "};")
-            (formatln out "static const uintL ~A_table_size = (sizeof(~A_table)/sizeof(struct c_lisp_pair))-1;" c-name c-name)
-            (formatln out "static ~A ~A (object a) {" c-type c-name)
-            (formatln out "  unsigned int index;")
-            (formatln out " restart_~A:" c-name)
-            (formatln out "  if (integerp(a)) return (~A)I_to_L(a);" c-type)
-            (when default
-              (let ((need-ifdef (and (not enum-p) (stringp default))))
-                (when need-ifdef (formatln out " #ifdef ~A" default))
-                (formatln out "  else if (missingp(a)) return ~A;" default)
-                (when need-ifdef (formatln out " #endif"))))
-            (formatln out "  else {")
-            (formatln out "    for (index = 0; index < ~A_table_size; index++)"
+            (formatln out "static const c_lisp_map_t ~A_map = {" c-name)
+            (formatln out "  ~A_table," c-name)
+            (formatln out "  (sizeof(~A_table)/sizeof(c_lisp_pair_t))-1,"
                       c-name)
-            (formatln out "      if (eq(a,*~A_table[index].l_const))" c-name)
-            (formatln out "        return (~A)~A_table[index].c_const;"
-                      c-type c-name)
-            (formatln out "    pushSTACK(NIL); pushSTACK(a);")
-            (formatln out "    pushSTACK(O(~A));" type-tag)
-            (formatln out "    pushSTACK(O(~A));" type-tag)
-            (formatln out "    pushSTACK(a); pushSTACK(TheSubr(subr_self)->name);")
-            (formatln out "    check_value(type_error,GETTEXT(\"~~S: ~~S is not of type ~~S\"));")
-            (formatln out "    a = value1; goto restart_~A;" c-name)
-            (formatln out "  }")
-            (formatln out "}")
-            (formatln out "static object ~A_reverse (~A a) {" c-name c-type)
-            (formatln out "  unsigned int index = 0;")
-            (formatln out "  for (; index < ~A_table_size; index++)" c-name)
-            (formatln out "    if (a == ~A_table[index].c_const)" c-name)
-            (formatln out "      return *~A_table[index].l_const;" c-name)
-            (when need-default
-              (unless enum-p (formatln out " #ifdef ~A" default))
-              (formatln out "  if (a == ~A) return NIL;" default)
-              (unless enum-p (formatln out " #endif")))
-            (if (stringp reverse)
-                (formatln out "  return ~A(a);" reverse)
-                (formatln out "  NOTREACHED;"))
-            (formatln out "}")
+            (cond ((and (not enum-p) (stringp default))
+                   (formatln out "# if defined(~A)" default)
+                   (formatln out "  ~A,true," default)
+                   (formatln out "# else")
+                   (formatln out "  0,false,")
+                   (formatln out "# endif"))
+                  (t (formatln out "  ~A,true," (or default 0))))
+            (formatln out "  true,") ; use_default_function_p
+            (formatln out "  ~S" c-name)
+            (formatln out "};")
+            (formatln out "#define ~A(a) (~A)map_lisp_to_c(a,&~A_map)"
+                      c-name c-type c-name)
+            (formatln out "#define ~A_reverse(a) map_c_to_lisp(a,&~A_map)"
+                      c-name c-name)
             (when bitmasks
-              (formatln out "static object ~A_to_list (~A a) {" c-name c-type)
-              (formatln out "  int count = 0;")
-              (formatln out "  unsigned int index = 0;")
-              (formatln out "  for (; index < ~A_table_size; index++) {" c-name)
-              (formatln out "    unsigned int c_const = ~A_table[index].c_const;" c-name)
-              (formatln out "    if (c_const && (c_const == (a & c_const))) {")
-              (formatln out "      pushSTACK(*~A_table[index].l_const);" c-name)
-              (formatln out "      count++;")
-              (formatln out "      a &= ~~c_const;") ; clear this bit
-              (formatln out "    }")
-              (formatln out "  }")
-              ;; not all bits accounted for:
-              (formatln out "  if (a) { pushSTACK(fixnum(a)); count++; }")
-              (formatln out "  return listof(count);")
-              (formatln out "}")
-              (formatln out "static ~A ~A_from_list (object a) {" c-type c-name)
-              (formatln out "  ~A ret = 0;" c-type)
-              (formatln out "  pushSTACK(a);")
-              (formatln out "  for (; !endp(STACK_0); STACK_0 = Cdr(STACK_0))")
-              (formatln out "    ret |= ~A(Car(STACK_0));" c-name)
-              (formatln out "  skipSTACK(1);")
-              (formatln out "  return ret;")
-              (formatln out "}")
-              (formatln out "static ~A ~A_parse (object a) {" c-type c-name)
-              (formatln out "  if (integerp(a)) return (~A)I_to_L(a);" c-type)
-              (formatln out "  if (listp(a)) return ~A_from_list(a);" c-name)
-              (formatln out "  return ~A(a);" c-name)
-              (formatln out "}"))))
+              (formatln out "#define ~A_to_list(a) map_c_to_list(a,&~A_map)"
+                        c-name c-name)
+              (formatln out "#define ~A_from_list(a) map_list_to_c(a,&~A_map)"
+                        c-name c-name))))
     (newline out)))
 
 (defun print-tables-2 (out)
@@ -1127,9 +1078,9 @@ commas and parentheses."
           (format out "  LISPFUN_F~A" (fundef-lispfun fd sig)))))
     (formatln out "  0")
     (formatln out "};")
-    (formatln out "uintC module__~A__subr_tab_size = (sizeof(struct ~A_t)-varobjects_misaligned-sizeof(int))/sizeof(subr_t);" *module-name* subr-tab)
+    (formatln out "uintC ~A_size = (sizeof(struct ~A_t)-varobjects_misaligned-sizeof(int))/sizeof(subr_t);" subr-tab subr-tab)
     (newline out)
-    (formatln out "struct {")
+    (formatln out "struct ~A_t {" subr-tab-initdata)
     (loop :for fd :across *fundefs*
       :do (with-conditional (out (fundef-cond-stack fd))
             (format out "  subr_initdata_t _~A;" (fundef-tag fd))))
@@ -1140,7 +1091,7 @@ commas and parentheses."
             (format out "  { ~S, ~S }," (fundef-pack fd) (fundef-name fd))))
     (formatln out "  0")
     (formatln out "};") (newline out)
-    ; Emit the decl first, to avoid "gcc -missing-declarations" warnings.
+    ;; Emit the decl first, to avoid "gcc -missing-declarations" warnings.
     (formatln out "void ~A (module_t* module);" *init-1-name*)
     (formatln out "void ~A (module_t* module)" *init-1-name*)
     (formatln out "{")
@@ -1158,14 +1109,14 @@ commas and parentheses."
     (formatln out "}")
     (when *init-2-name*         ; no init2 => define a dummy
       (newline out)
-      ; Emit the decl first, to avoid "gcc -missing-declarations" warnings.
+      ;; Emit the decl first, to avoid "gcc -missing-declarations" warnings.
       (formatln out "void ~A (module_t* module);" *init-2-name*)
       (formatln out "void ~A (module_t* module)" *init-2-name*)
       (formatln out "{")
       (formatln out "}"))
     (when *fini-name*           ; no fini-func => define a dummy
       (newline out)
-      ; Emit the decl first, to avoid "gcc -missing-declarations" warnings.
+      ;; Emit the decl first, to avoid "gcc -missing-declarations" warnings.
       (formatln out "void ~A (module_t* module);" *fini-name*)
       (formatln out "void ~A (module_t* module)" *fini-name*)
       (formatln out "{")
@@ -1196,7 +1147,7 @@ commas and parentheses."
     (setq *lines* (read-all-input in)))
   (format t "~:D line~:P~%" (length *lines*))
   (parse *lines*)
-  (format t "~%;; ~S: ~:D object~:P, ~:D DEFUN~:P~[~:;~:* (~:d emulated)~]~
+  (format t "~&;; ~S: ~:D object~:P, ~:D DEFUN~:P~[~:;~:* (~:d emulated)~]~
 ~[~*~:;~:*, ~:D DEFVAR~:P (~:D init~:P)~]~%;; packages: ~S~%"
           'modprep (length *objdefs*) (length *fundefs*)
           *emulation-count* (length *vardefs*) (length *varinits*)
